clear, clc, close all
set(0,'DefaultLineLineWidth',1.5)
addpath 'C:\Program Files (x86)\REFPROP\'

%% DESIGN PARAMETERS
%low stage refrigerant
ref1='co2butane 0.03-0.97.mix';
%high stage refrigerant
ref2='co2pentane 0.02-0.98.mix';

%condenser area
Ac=22; %m^2
%cascade heat exchanger area
Ache=12; %m^2
%evaporator area
Ae=13; %m^2
%low stage internal heat exchanger area
Aihxls=1.1; %m^2

%low stage compressor size
Vcls=145; %m^3/h
%high stage compressor size
Vchs=150; %m^3/h
%number of operating compressors
ncomp=2;

%% PROCESS PARAMETERS
%hot water inlet temperature
T11=60+273.15; %K
%heat source inlet temperature
T1=40+273.15; %K

%hot water mass flow rate
m2=1.19; %kg/s
%heat source mass flow rate
m1=3.76; %kg/s

%degree of superheating
Tsup=12; %K
%temperature difference between the stages
Tdiff=8; %K

%% PROCESS CONSTANTS
%global heat transfer coefficient in the high stage condenser
Uc=1400; %W/m^2*K
%global heat transfer coefficient in the cascade heat exchanger
Uche=1800; %W/m^2*K
%global heat transfer coefficient in the low stage evaporator
Ue=1600; %W/m^2*K
%global heat transfer coefficient in the internal heat exchanger
Uihx=1000; %W/m^2*K

%% INITIAL GUESS VALUES
%high stage condensation temperature
Tchs=80+273.15; %K
%low stage condensation temperature
Tcls=52+273.15; %K
%low stage evaporation temperature
Tels=30+273.15; %K

%% SYSTEM SOLUTION
%system equations solution
[Ts,E]=fsolve(@eqn,[Tchs,Tcls,Tels],[],Tsup,Tsup,Tdiff,T1,T11,m1,m2,Uc,Ac,Uche,Ache,Ue,Ae,Vcls,Vchs,ncomp,ref1,ref2);

%high stage condensation temperature
Tchs=Ts(1); %K
%low stage condensation temperature
Tcls=Ts(2); %K
%low stage evaporation temperature
Tels=Ts(3); %K
%error in the condenser LMTD
Ec=E(1); %K
%error in the cascade heat exchanger LMTD
Eche=E(2); %K
%error in the evaporator
Ee=E(3); %K

clc
if abs(Ec)>0.1 || abs(Eche)>0.1 || abs(Ee)>0.1
    error('LMTDs not in agreement')
else
    fprintf('Equations solved \n')
end

%% CYCLES COMPUTATIONS
%high stage condenser pressure
Pchs=refpropm('P','T',Tchs,'Q',0,ref2)/100; %bar
%low stage condenser pressure
Pcls=refpropm('P','T',Tcls,'Q',0,ref1)/100; %bar
%low stage evaporator pressure
Pels=refpropm('P','T',Tels,'Q',1,ref1)/100; %bar
%low stage compression ratio
rls=Pcls/Pels;
%low stage isoentropic efficiency
etaSls=-0.00000461*rls^6+0.00027131*rls^5-0.00628605*rls^4+0.07370258*rls^3-0.46054399*rls^2+1.40653347*rls-0.87811477;
%low stage volumetric efficiency
etaVls=0.001*rls^2-0.0487*rls+0.9979;

%low stage compressor inlet temperature
T4=Tels+Tsup; %K
%low stage compressor inlet enthalpy
H4=refpropm('H','T',T4,'P',Pels*100,ref1)/1000; %kJ/kg
%low stage compressor inlet density
D4=refpropm('D','T',T4,'P',Pels*100,ref1); %kg/m^3
%low stage compressor inlet entropy
S4=refpropm('S','T',T4,'P',Pels*100,ref1)/1000; %kJ/kg*K
%low stage condenser inlet enthalpy (isentropic)
H5s=refpropm('H','P',Pcls*100,'S',S4*1000,ref1)/1000; %kJ/kg
%low stage condenser inlet enthalpy (real)
H5=H4+(H5s-H4)/etaSls; %kJ/kg
%low stage condenser inlet temperature
T5=refpropm('T','P',Pcls*100,'H',H5*1000,ref1); %K
%low stage internal heat exchanger enthalpy change
DHihxls=H4-refpropm('H','T',Tels,'Q',1,ref1)/1000; %kJ/kg
%low stage valve inlet enthalpy
H6=refpropm('H','T',Tcls,'Q',0,ref1)/1000-DHihxls; %kJ/kg
%low stage valve inlet temperature
T6=refpropm('T','P',Pcls*100,'H',H6*1000,ref1); %K
%low stage degree of subcooling
Tsubls=Tcls-T6; %K
%low stage evaporator inlet enthalpy
H3=H6; %kJ/kg
%low stage evaporator inlet temperature
T3=refpropm('T','P',Pels*100,'H',H3*1000,ref1); %K

%low stage refrigerant mass flow rate
mrefls=D4*Vcls/3600*etaVls*ncomp; %kg/s

%low stage compressor power demand
Wcls=(H5-H4)*mrefls; %kW
%low stage evaporator capacity
Qe=(refpropm('H','P',Pels*100,'Q',1,ref1)/1000-H3)*mrefls; %kW
%cascade heat exchanger capacity
Qche=(H5-refpropm('H','P',Pcls*100,'Q',0,ref1)/1000)*mrefls; %kW

%low stage coefficient of performance
COPls=Qche/Wcls; 

%heat source specific heat capacity
cp1=refpropm('C','T',T1,'P',20*100,'water')/1000; %kJ/kg*K
%heat source outlet temperature
T2=T1-Qe/cp1/m1; %K
%logarithmic mean temperature difference in the low stage evaporator
LMTDe=Qe*1000/Ue/Ae; %K

%high stage compressor inlet temperature
T8=T5-Tdiff; %K
%high stage evaporation temperature
Tehs=T8-Tsup; %K

%high stage evaporator pressure
Pehs=refpropm('P','T',Tehs,'Q',1,ref2)/100; %bar
%high stage compression ratio
rhs=Pchs/Pehs;
%high stage isoentropic efficiency
etaShs=-0.00000461*rhs^6+0.00027131*rhs^5-0.00628605*rhs^4+0.07370258*rhs^3-0.46054399*rhs^2+1.40653347*rhs-0.87811477;
%high stage volumetric efficiency
etaVhs=0.001*rhs^2-0.0487*rhs+0.9979;

%high stage compressor inlet enthalpy
H8=refpropm('H','T',T8,'P',Pehs*100,ref2)/1000; %kJ/kg
%high stage compressor inlet density
D8=refpropm('D','T',T8,'P',Pehs*100,ref2); %kg/m^3
%high stage compressor inlet entropy
S8=refpropm('S','T',T8,'P',Pehs*100,ref2)/1000; %kJ/kg*K
%high stage condenser inlet enthalpy (isentropic)
H9s=refpropm('H','P',Pchs*100,'S',S8*1000,ref2)/1000; %kJ/kg
%high stage condenser inlet enthalpy (real)
H9=H8+(H9s-H8)/etaShs; %kJ/kg
%high stage condenser inlet temperature
T9=refpropm('T','P',Pchs*100,'H',H9*1000,ref2); %K

%high stage refrigerant mass flow rate
mrefhs=D8*Vchs/3600*etaVhs*ncomp; %kg/s

%high stage evaporator inlet enthalpy
H7=refpropm('H','P',Pehs*100,'Q',1,ref2)/1000-Qche/mrefhs; %kJ/kg
%high stage evaporator inlet temperature
T7=refpropm('T','P',Pehs*100,'H',H7*1000,ref2); %K
%high stage valve inlet enthalpy
H10=H7; %kJ/kg
%high stage valve inlet temperature
T10=refpropm('T','P',Pchs*100,'H',H10*1000,ref2); %K
%high stage intermediate subcooling temperature
T10b=T11+5; %K
%high stage internal heat exchanger enthalpy change
DHihxhs=refpropm('H','T',T10b,'P',Pchs*100,ref2)/1000-H10; %kJ/kg
%high stage degree of subcooling
Tsubhs=Tchs-T10; %K

%high stage compressor power demand
Wchs=(H9-H8)*mrefhs; %kW
%high stage condenser capacity
Qc=(H9-refpropm('H','T',T10b,'P',Pchs*100,ref2)/1000)*mrefhs; %kW

%high stage coefficient of performance
COPhs=Qc/Wchs;

%hot water specific heat capacity
cp2=refpropm('C','T',T11,'P',20*100,'water')/1000; %kJ/kg*K
%hot water outlet temperature
T12=T11+Qc/cp2/m2; %K
%logarithmic mean temperature difference in the high stage condenser
LMTDc=Qc*1000/Uc/Ac; %K

%logarithmic mean temperature difference in the cascade heat exchanger
LMTDche=Qche*1000/Uche/Ache; %K

%overall compressor power demand
Wc=Wchs+Wcls; %kW
%overall coefficient of performance
COP=Qc/Wc;

%heat source outlet temperature after energy integration
T2a=T2+mrefhs*(refpropm('H','T',T10b,'P',Pchs*100,ref2)/1000-H10)/m1/cp1; %K

%% DISPLAYS
fprintf('The heat pump capacity is: %0.1f',Qc),fprintf('kW\n')
fprintf('The heat pump power consumption is: %0.1f',Wc),fprintf('kW\n')
if round(COP,1)<3.6
    warning('COP out of specification')
end
fprintf('The overall COP is: %0.3f\n',COP)
if T12<100+273.15
    warning('Hot water temperature out of specification')
end
fprintf('The hot water temperature is: %0.2f',T12-273.15),fprintf('°C\n')
if LMTDc<4 || LMTDc>12
    warning('LMTDc out of optimal range')
end
if LMTDche<4 || LMTDche>12
    warning('LMTDche out of optimal range')
end
if LMTDe<4 || LMTDe>12
    warning('LMTDe out of optimal range')
end

%% T-S DIAGRAM
figure(1)
hold on
plot([refpropm('S','T',T3,'P',Pels*100,ref1),refpropm('S','P',Pels*100,'Q',1,ref1)]/1000,[T3,Tels]-273.15,'b-')
plot([refpropm('S','P',Pels*100,'Q',1,ref1),refpropm('S','T',T4,'P',Pels*100,ref1)]/1000,[Tels,T4]-273.15,'b-')
plot([refpropm('S','T',T4,'P',Pels*100,ref1),refpropm('S','T',T4,'P',Pels*100,ref1)]/1000,[T4,refpropm('T','P',Pcls*100,'S',refpropm('S','T',T4,'P',Pels*100,ref1),ref1)]-273.15,'b--')
plot([refpropm('S','T',T4,'P',Pels*100,ref1),refpropm('S','T',T5,'P',Pcls*100,ref1)]/1000,[T4,T5]-273.15,'b-')
plot([refpropm('S','T',T5,'P',Pcls*100,ref1),refpropm('S','P',Pcls*100,'Q',1,ref1)]/1000,[T5,refpropm('T','P',Pcls*100,'Q',1,ref1)]-273.15,'b-')
plot([refpropm('S','P',Pcls*100,'Q',1,ref1),refpropm('S','P',Pcls*100,'Q',0,ref1)]/1000,[refpropm('T','P',Pcls*100,'Q',1,ref1),Tcls]-273.15,'b-')
plot([refpropm('S','P',Pcls*100,'Q',0,ref1),refpropm('S','T',T6,'P',Pcls*100,ref1)]/1000,[Tcls,T6]-273.15,'b-')
plot([refpropm('S','T',T6,'P',Pcls*100,ref1),refpropm('S','T',T3,'P',Pels*100,ref1)]/1000,[T6,T3]-273.15,'b-')
plot([refpropm('S','T',T7,'P',Pehs*100,ref2),refpropm('S','P',Pehs*100,'Q',1,ref2)]/1000,[T7,Tehs]-273.15,'c-')
plot([refpropm('S','P',Pehs*100,'Q',1,ref2),refpropm('S','T',T8,'P',Pehs*100,ref2)]/1000,[Tehs,T8]-273.15,'c-')
plot([refpropm('S','T',T8,'P',Pehs*100,ref2),refpropm('S','T',T8,'P',Pehs*100,ref2)]/1000,[T8,refpropm('T','P',Pchs*100,'S',refpropm('S','T',T8,'P',Pehs*100,ref2),ref2)]-273.15,'c--')
plot([refpropm('S','T',T8,'P',Pehs*100,ref2),refpropm('S','T',T9,'P',Pchs*100,ref2)]/1000,[T8,T9]-273.15,'c-')
plot([refpropm('S','T',T9,'P',Pchs*100,ref2),refpropm('S','P',Pchs*100,'Q',1,ref2)]/1000,[T9,refpropm('T','P',Pchs*100,'Q',1,ref2)]-273.15,'c-')
plot([refpropm('S','P',Pchs*100,'Q',1,ref2),refpropm('S','P',Pchs*100,'Q',0,ref2)]/1000,[refpropm('T','P',Pchs*100,'Q',1,ref2),Tchs]-273.15,'c-')
plot([refpropm('S','P',Pchs*100,'Q',0,ref2),refpropm('S','T',T10,'P',Pchs*100,ref2)]/1000,[Tchs,T10]-273.15,'c-')
plot([refpropm('S','T',T10,'P',Pchs*100,ref2),refpropm('S','T',T7,'P',Pehs*100,ref2)]/1000,[T10,T7]-273.15,'c-')
scatter(refpropm('S','T',T3,'P',Pels*100,ref1)/1000,T3-273.15,'ko','filled')
scatter(refpropm('S','P',Pels*100,'Q',1,ref1)/1000,Tels-273.15,'ko','filled')
scatter(refpropm('S','T',T4,'P',Pels*100,ref1)/1000,T4-273.15,'ko','filled')
scatter(refpropm('S','T',T5,'P',Pcls*100,ref1)/1000,T5-273.15,'ko','filled')
scatter(refpropm('S','T',T4,'P',Pels*100,ref1)/1000,refpropm('T','P',Pcls*100,'S',refpropm('S','T',T4,'P',Pels*100,ref1),ref1)-273.15,'ko','filled')
scatter(refpropm('S','P',Pcls*100,'Q',0,ref1)/1000,Tcls-273.15,'ko','filled')
scatter(refpropm('S','T',T6,'P',Pcls*100,ref1)/1000,T6-273.15,'ko','filled')
scatter(refpropm('S','T',T7,'P',Pehs*100,ref2)/1000,T7-273.15,'ko','filled')
scatter(refpropm('S','P',Pehs*100,'Q',1,ref2)/1000,Tehs-273.15,'ko','filled')
scatter(refpropm('S','T',T8,'P',Pehs*100,ref2)/1000,T8-273.15,'ko','filled')
scatter(refpropm('S','T',T9,'P',Pchs*100,ref2)/1000,T9-273.15,'ko','filled')
scatter(refpropm('S','T',T8,'P',Pehs*100,ref2)/1000,refpropm('T','P',Pchs*100,'S',refpropm('S','T',T8,'P',Pehs*100,ref2),ref2)-273.15,'ko','filled')
scatter(refpropm('S','P',Pchs*100,'Q',0,ref2)/1000,Tchs-273.15,'ko','filled')
scatter(refpropm('S','T',T10b,'P',Pchs*100,ref2)/1000,T10b-273.15,'ko','filled')
scatter(refpropm('S','T',T10,'P',Pchs*100,ref2)/1000,T10-273.15,'ko','filled')
Tr1=linspace(10+273.15,refpropm('T','C',0,'',0,ref1),30); %K
Tr2=linspace(10+273.15,refpropm('T','C',0,'',0,ref2),30); %K
Sls=zeros(2,length(Tr1));
Svs=zeros(2,length(Tr1));
for i=1:length(Tr1)
    Sls(1,i)=refpropm('S','T',Tr1(i),'Q',0,ref1)/1000; %kJ/kg*K
    Svs(1,i)=refpropm('S','T',Tr1(i),'Q',1,ref1)/1000; %kJ/kg*K
    Sls(2,i)=refpropm('S','T',Tr2(i),'Q',0,ref2)/1000; %kJ/kg*K
    Svs(2,i)=refpropm('S','T',Tr2(i),'Q',1,ref2)/1000; %kJ/kg*K
end
plot(Sls(1,:),Tr1-273.15,'g-.',Svs(1,:),Tr1-273.15,'g.-',Sls(2,:),Tr2-273.15,'y.-',Svs(2,:),Tr2-273.15,'y.-')
scatter(refpropm('S','C',0,'',0,ref1)/1000,refpropm('T','C',0,'',0,ref1)-273.15,'kp','filled')
scatter(refpropm('S','C',0,'',0,ref2)/1000,refpropm('T','C',0,'',0,ref2)-273.15,'kp','filled')
grid on
xlabel('Specific Entropy [kJ/(kg*K)]')
ylabel('Temperature [°C]')
title('T-S Diagram Cascade HP')
legend('Low stage','','','','','','','','High Stage','','','','','','','','Location','NorthWest')
text(refpropm('S','T',T3,'P',Pels*100,ref1)/1000-0.1,T3-273.15,cellstr('3'))
text(refpropm('S','P',Pels*100,'Q',1,ref1)/1000+0.05,Tels-273.15,cellstr('4a'))
text(refpropm('S','T',T4,'P',Pels*100,ref1)/1000+0.05,T4-273.15,cellstr('4'))
text(refpropm('S','T',T5,'P',Pcls*100,ref1)/1000+0.05,T5-273.15,cellstr('5'))
text(refpropm('S','T',T4,'P',Pels*100,ref1)/1000-0.16,refpropm('T','P',Pcls*100,'S',refpropm('S','T',T4,'P',Pels*100,ref1),ref1)-273.15,cellstr('5s'))
text(refpropm('S','P',Pcls*100,'Q',0,ref1)/1000-0.16,Tcls-273.15,cellstr('6a'))
text(refpropm('S','T',T6,'P',Pcls*100,ref1)/1000-0.1,T6-273.15,cellstr('6'))
text(refpropm('S','T',T7,'P',Pehs*100,ref2)/1000-0.1,T7-273.15,cellstr('7'))
text(refpropm('S','P',Pehs*100,'Q',1,ref2)/1000+0.05,Tehs-273.15,cellstr('8a'))
text(refpropm('S','T',T8,'P',Pehs*100,ref2)/1000+0.05,T8-273.15,cellstr('8'))
text(refpropm('S','T',T9,'P',Pchs*100,ref2)/1000+0.05,T9-273.15,cellstr('9'))
text(refpropm('S','T',T8,'P',Pehs*100,ref2)/1000-0.16,refpropm('T','P',Pchs*100,'S',refpropm('S','T',T8,'P',Pehs*100,ref2),ref2)-273.15,cellstr('9s'))
text(refpropm('S','P',Pchs*100,'Q',0,ref2)/1000-0.22,Tchs-273.15+4,cellstr('10b'))
text(refpropm('S','T',T10b,'P',Pchs*100,ref2)/1000-0.22,T10b-273.15,cellstr('10a'))
text(refpropm('S','T',T10,'P',Pchs*100,ref2)/1000-0.16,T10-273.15+4,cellstr('10'))
text(refpropm('S','C',0,'',0,ref1)/1000,refpropm('T','C',0,'',0,ref1)-273.15+6,cellstr('CP_1'))
text(refpropm('S','C',0,'',0,ref2)/1000,refpropm('T','C',0,'',0,ref2)-273.15+6,cellstr('CP_2'))
xlim([refpropm('S','T',T7,'P',Pehs*100,ref2)/1000-0.5,refpropm('S','T',T5,'P',Pels*100,ref1)/1000+0.5])
ylim([10,210])

%% P-H DIAGRAM
figure(2)
hold on
plot([H3,H4],[Pels,Pels],'b-')
plot([H4,H5],[Pels,Pcls],'b-')
plot([H4,H5s],[Pels,Pcls],'b--')
plot([H5,H6],[Pcls,Pcls],'b-')
plot([H6,H3],[Pcls,Pels],'b-')
plot([H7,H8],[Pehs,Pehs],'c-')
plot([H8,H9],[Pehs,Pchs],'c-')
plot([H8,H9s],[Pehs,Pchs],'c--')
plot([H9,H10],[Pchs,Pchs],'c-')
plot([H10,H7],[Pchs,Pehs],'c-')
scatter(H3,Pels,'ko','filled')
scatter(refpropm('H','P',Pels*100,'Q',1,ref1)/1000,Pels,'ko','filled')
scatter(H4,Pels,'ko','filled')
scatter(H5,Pcls,'ko','filled')
scatter(H5s,Pcls,'ko','filled')
scatter(refpropm('H','P',Pcls*100,'Q',0,ref1)/1000,Pcls,'ko','filled')
scatter(H6,Pcls,'ko','filled')
scatter(H7,Pehs,'ko','filled')
scatter(refpropm('H','P',Pehs*100,'Q',1,ref2)/1000,Pehs,'ko','filled')
scatter(H8,Pehs,'ko','filled')
scatter(H9,Pchs,'ko','filled')
scatter(H9s,Pchs,'ko','filled')
scatter(refpropm('H','P',Pchs*100,'Q',0,ref2)/1000,Pchs,'ko','filled')
scatter(refpropm('H','T',T10b,'P',Pchs*100,ref2)/1000,Pchs,'ko','filled')
scatter(H10,Pchs,'ko','filled')
Pr1=logspace(log10(2),log10(refpropm('P','C',0,'',0,ref1)/100-2),30); %bar
Pr2=logspace(log10(2),log10(refpropm('P','C',0,'',0,ref2)/100-2),30); %bar
Hls=zeros(2,length(Pr1));
Hvs=zeros(2,length(Pr1));
for i=1:length(Pr1)
    Hls(1,i)=refpropm('H','P',Pr1(i)*100,'Q',0,ref1)/1000; %kJ/kg
    Hvs(1,i)=refpropm('H','P',Pr1(i)*100,'Q',1,ref1)/1000; %kJ/kg
    Hls(2,i)=refpropm('H','P',Pr2(i)*100,'Q',0,ref2)/1000; %kJ/kg
    Hvs(2,i)=refpropm('H','P',Pr2(i)*100,'Q',1,ref2)/1000; %kJ/kg
end
plot(Hls(1,:),Pr1,'g-.',Hvs(1,:),Pr1,'g.-',Hls(2,:),Pr2,'y.-',Hvs(2,:),Pr2,'y.-')
scatter(refpropm('H','C',0,'',0,ref1)/1000,refpropm('P','C',0,'',0,ref1)/100,'kp','filled')
scatter(refpropm('H','C',0,'',0,ref2)/1000,refpropm('P','C',0,'',0,ref2)/100,'kp','filled')
grid on
set(gca,'YScale','log')
xlabel('Specific Enthalpy [kJ/kg]')
ylabel('Pressure [bar]')
title('P-H Diagram Cascade HP')
legend('Low Stage','','','','','High Stage','Location','NorthWest')
text(H3-5,10^(log10(Pels)-0.05),cellstr('3'))
text(refpropm('H','P',Pels*100,'Q',1,ref1)/1000-15,10^(log10(Pels)-0.05),cellstr('4a'))
text(H4,10^(log10(Pels)-0.05),cellstr('4'))
text(H5,10^(log10(Pcls)+0.05),cellstr('5'))
text(H5s-20,10^(log10(Pcls)+0.05),cellstr('5s'))
text(refpropm('H','P',Pcls*100,'Q',0,ref1)/1000-5,10^(log10(Pcls)+0.05),cellstr('6a'))
text(H6-10,10^(log10(Pcls)+0.05),cellstr('6'))
text(H7-5,10^(log10(Pehs)-0.05),cellstr('7'))
text(refpropm('H','P',Pehs*100,'Q',1,ref2)/1000-15,10^(log10(Pehs)-0.05),cellstr('8a'))
text(H8,10^(log10(Pehs)-0.05),cellstr('8'))
text(H9,10^(log10(Pchs)+0.05),cellstr('9'))
text(H9s-20,10^(log10(Pchs)+0.05),cellstr('9s'))
text(refpropm('H','P',Pchs*100,'Q',0,ref2)/1000-40,10^(log10(Pchs)+0.05),cellstr('10a'))
text(refpropm('H','T',T10b,'P',Pchs*100,ref2)/1000+10,10^(log10(Pchs)-0.05),cellstr('10b'))
text(H10-20,10^(log10(Pchs)+0.05),cellstr('10'))
text(refpropm('H','C',0,'',0,ref1)/1000-40,refpropm('P','C',0,'',0,ref1)/100-6,cellstr('CP_1'))
text(refpropm('H','C',0,'',0,ref2)/1000-40,refpropm('P','C',0,'',0,ref2)/100-6,cellstr('CP_2'))
xlim([H7-100,H5+100])
ylim([2,45])

%% MAIN HEAT EXCHANGERS INTERNAL MODELING
figure(3)

%number of slices
nslices=10;
%intermediate temperatures and LMTDs in the evaporator
[Tie,LMTDei]=evaporatormodel(nslices,T1,T2,Tels,T3,Pels,ref1);

subplot(2,3,1)
hold on
plot(linspace(0,Ae,nslices+1),Tie(1,:)-273.15,'ro-')
plot(linspace(0,Ae,nslices+1),Tie(2,:)-273.15,'bo-')
for i=1:nslices-1
    plot(Ae/nslices*[i,i],[0,100],'k--','LineWidth',1)
end
grid on
xlabel('Exchanger Area [m^2]')
ylabel('Temperature [°C]')
title('Evaporator Temperature Profiles')
legend('Heat Source side','Refrigerant side','Location','NorthWest')
xlim([0,Ae])
ylim([min(Tie(2,:))-4,max(Tie(1,:))+4]-273.15)

subplot(2,3,4)
hold on
plot(linspace(Ae/nslices/2,Ae-Ae/nslices/2,nslices),LMTDei,'go-')
for i=1:nslices-1
    plot(Ae/nslices*[i,i],[0,100],'k--','LineWidth',1)
end
grid on
xlabel('Exchanger Area [m^2]')
ylabel('LMTDe [K]')
title('Evaporator LMTD')
xlim([0,Ae])
ylim([min(LMTDei)-0.2,max(LMTDei)+0.2])

%number of slices
nslices=10;
%intermediate temperatures and LMTDs in the cascade heat exchanger
[Tiche,LMTDchei]=cascadeHXmodel(nslices,Tcls,T5,Pcls,ref1,T7,T8,Pehs,ref2);

subplot(2,3,2)
hold on
plot(linspace(0,Ache,nslices+1),Tiche(1,:)-273.15,'ro-')
plot(linspace(0,Ache,nslices+1),Tiche(2,:)-273.15,'bo-')
for i=1:nslices-1
    plot(Ache/nslices*[i,i],[0,200],'k--','LineWidth',1)
end
grid on
xlabel('Exchanger Area [m^2]')
ylabel('Temperature [°C]')
title('Cascade Heat Exchanger Temperature Profiles')
legend('Low Stage Refrigerant side','High Stage Refrigerant side','Location','NorthWest')
xlim([0,Ache])
ylim([min(Tiche(2,:))-4,max(Tiche(1,:))+4]-273.15)

subplot(2,3,5)
hold on
plot(linspace(Ache/nslices/2,Ache-Ache/nslices/2,nslices),LMTDchei,'go-')
for i=1:nslices-1
    plot(Ache/nslices*[i,i],[0,100],'k--','LineWidth',1)
end
grid on
xlabel('Exchanger Area [m^2]')
ylabel('LMTDche [K]')
title('Cascade Heat Exchanger LMTD')
xlim([0,Ache])
ylim([min(real(LMTDchei))-0.2,max(real(LMTDchei))+0.2])

%number of slices
nslices=10;
%intermediate temperatures and LMTDs in the condenser
[Tic,LMTDci]=condensermodel(nslices,T11,T12,T10b,T9,Pchs,ref2);

subplot(2,3,3)
hold on
plot(linspace(0,Ac,nslices+1),Tic(1,:)-273.15,'ro-')
plot(linspace(0,Ac,nslices+1),Tic(2,:)-273.15,'bo-')
for i=1:nslices-1
    plot(Ac/nslices*[i,i],[0,200],'k--','LineWidth',1)
end
grid on
xlabel('Exchanger Area [m^2]')
ylabel('Temperature [°C]')
title('Condenser Temperature Profiles')
legend('Hot Water side','Refrigerant side','Location','NorthWest')
xlim([0,Ac])
ylim([min(Tic(1,:))-4,max(Tic(2,:))+4]-273.15)

subplot(2,3,6)
hold on
plot(linspace(Ac/nslices/2,Ac-Ac/nslices/2,nslices),LMTDci,'go-')
for i=1:nslices-1
    plot(Ac/nslices*[i,i],[0,100],'k--','LineWidth',1)
end
grid on
xlabel('Exchanger Area [m^2]')
ylabel('LMTDc [K]')
title('Condenser LMTD')
xlim([0,Ac])
ylim([min(LMTDci)-0.2,max(LMTDci)+0.2])

%% INTERNAL HEAT EXCHANGER INTERNAL MODELLING
%number of slices
nslices=10;
%intermediate temperatures and LMTDs in the internal heat exchangers
[Tiihx,LMTDihxi]=internalHXmodel(nslices,T6,Tcls,Pcls,Tels,T4,Pels,ref1);

figure(4)

subplot(2,1,1)
hold on
plot(linspace(0,Aihxls,nslices+1),Tiihx(1,:)-273.15,'ro-')
plot(linspace(0,Aihxls,nslices+1),Tiihx(2,:)-273.15,'bo-')
for i=1:nslices-1
    plot(Aihxls/nslices*[i,i],[0,100],'k--','LineWidth',1)
end
grid on
xlabel('Exchanger Area [m^2]')
ylabel('Temperature [°C]')
title('Low Stage Internal Heat Exchanger Temperature Profiles')
legend('Hot Refrigerant side','Cold Refrigerant side','Location','NorthWest')
xlim([0,Aihxls])
ylim([min(Tiihx(2,:))-4,max(Tiihx(1,:))+4]-273.15)

subplot(2,1,2)
hold on
plot(linspace(Aihxls/nslices/2,Aihxls-Aihxls/nslices/2,nslices),LMTDihxi(1,:),'go-')
for i=1:nslices-1
    plot(Aihxls/nslices*[i,i],[0,100],'k--','LineWidth',1)
end
grid on
xlabel('Exchanger Area [m^2]')
ylabel('LMTDihx [K]')
title('Low Stage Internal Heat Exchanger LMTD')
xlim([0,Aihxls])
ylim([min(LMTDihxi(1,:))-0.2,max(LMTDihxi(1,:))+0.2])

%% CASCADE HEAT PUMP EQUATIONS
function err=eqn(Ts,Tsuphs,Tsupls,Tdiff,T1,T11,m1,m2,Uc,Ac,Uche,Ache,Ue,Ae,Vcls,Vchs,ncomp,ref1,ref2)

Tchs=Ts(1);
Tcls=Ts(2);
Tels=Ts(3);

%high stage condenser pressure
Pchs=refpropm('P','T',Tchs,'Q',0,ref2)/100; %bar
%low stage condenser pressure
Pcls=refpropm('P','T',Tcls,'Q',0,ref1)/100; %bar
%low stage evaporator pressure
Pels=refpropm('P','T',Tels,'Q',1,ref1)/100; %bar
%low stage compression ratio
rls=Pcls/Pels;
%low stage isoentropic efficiency
etaSls=-0.00000461*rls^6+0.00027131*rls^5-0.00628605*rls^4+0.07370258*rls^3-0.46054399*rls^2+1.40653347*rls-0.87811477;
%low stage volumetric efficiency
etaVls=0.001*rls^2-0.0487*rls+0.9979;

%low stage compressor inlet temperature
T4=Tels+Tsupls; %K
%low stage compressor inlet enthalpy
H4=refpropm('H','T',T4,'P',Pels*100,ref1)/1000; %kJ/kg
%low stage compressor inlet density
D4=refpropm('D','T',T4,'P',Pels*100,ref1); %kg/m^3
%low stage compressor inlet entropy
S4=refpropm('S','T',T4,'P',Pels*100,ref1)/1000; %kJ/kg*K
%low stage condenser inlet enthalpy (isentropic)
H5s=refpropm('H','P',Pcls*100,'S',S4*1000,ref1)/1000; %kJ/kg
%low stage condenser inlet enthalpy (real)
H5=H4+(H5s-H4)/etaSls; %kJ/kg
%low stage condenser inlet temperature
T5=refpropm('T','P',Pcls*100,'H',H5*1000,ref1); %K
%internal heat exchanger enthalpy change
DHihx=H4-refpropm('H','T',Tels,'Q',1,ref1)/1000; %kJ/kg
%low stage valve inlet enthalpy
H6=refpropm('H','T',Tcls,'Q',0,ref1)/1000-DHihx; %kJ/kg
%low stage evaporator inlet enthalpy
H3=H6; %kJ/kg
%low stage evaporator inlet temperature
T3=refpropm('T','P',Pels*100,'H',H3*1000,ref1); %K

%low stage refrigerant mass flow rate
mrefls=D4*Vcls/3600*etaVls*ncomp; %kg/s

%evaporator capacity
Qe=(refpropm('H','P',Pels*100,'Q',1,ref1)/1000-H3)*mrefls; %kW
%cascade heat exchanger capacity
Qche=(H5-refpropm('H','P',Pcls*100,'Q',0,ref1)/1000)*mrefls; %kW

%cold water specific heat capacity
cp1=refpropm('C','T',T1,'P',20*100,'water')/1000; %kJ/kg*K
%cold water outlet temperature
T2=T1-Qe/cp1/m1; %K
%logarithmic mean temperature difference in the low stage evaporator
LMTDe=Qe*1000/Ue/Ae; %K
%error in the low stage evaporator
Ee=LMTDe-((T1-Tels)-(T2-T3))/log((T1-Tels)/(T2-T3)); %K

%high stage compressor inlet temperature
T8=T5-Tdiff; %K
%high stage evaporation temperature
Tehs=T8-Tsuphs; %K

%high stage evaporator pressure
Pehs=refpropm('P','T',Tehs,'Q',1,ref2)/100; %bar
%high stage compression ratio
rhs=Pchs/Pehs;
%high stage isoentropic efficiency
etaShs=-0.00000461*rhs^6+0.00027131*rhs^5-0.00628605*rhs^4+0.07370258*rhs^3-0.46054399*rhs^2+1.40653347*rhs-0.87811477;
%high stage volumetric efficiency
etaVhs=0.001*rhs^2-0.0487*rhs+0.9979;

%high stage compressor inlet enthalpy
H8=refpropm('H','T',T8,'P',Pehs*100,ref2)/1000; %kJ/kg
%high stage compressor inlet density
D8=refpropm('D','T',T8,'P',Pehs*100,ref2); %kg/m^3
%high stage compressor inlet entropy
S8=refpropm('S','T',T8,'P',Pehs*100,ref2)/1000; %kJ/kg*K
%high stage condenser inlet enthalpy (isentropic)
H9s=refpropm('H','P',Pchs*100,'S',S8*1000,ref2)/1000; %kJ/kg
%high stage condenser inlet enthalpy (real)
H9=H8+(H9s-H8)/etaShs; %kJ/kg
%high stage condenser inlet temperature
T9=refpropm('T','P',Pchs*100,'H',H9*1000,ref2); %K

%high stage refrigerant mass flow rate
mrefhs=D8*Vchs/3600*etaVhs*ncomp; %kg/s

%high stage evaporator inlet enthalpy
H7=refpropm('H','P',Pehs*100,'Q',1,ref2)/1000-Qche/mrefhs; %kJ/kg
%high stage evaporator inlet temperature
T7=refpropm('T','P',Pehs*100,'H',H7*1000,ref2); %K

%condenser capacity
Qc=(H9-refpropm('H','T',T11+5,'P',Pchs*100,ref2)/1000)*mrefhs; %kW

%hot water specific heat capacity
cp2=refpropm('C','T',T11,'P',20*100,'water')/1000; %kJ/kg*K
%hot water outlet temperature
T12=T11+Qc/cp2/m2; %K
%logarithmic mean temperature difference in the high stage condenser
LMTDc=Qc*1000/Uc/Ac; %K
%error in the high stage condenser
Ec=LMTDc-((T9-T12)-(5))/log((T9-T12)/(5)); %K

%logarithmic mean temperature difference in the cascade heat exchanger
LMTDche=Qche*1000/Uche/Ache; %K
%error in the cascade heat exchanger
Eche=LMTDche-((T5-T8)-(Tcls-T7))/log((T5-T8)/(Tcls-T7)); %K

err=[Ec;Eche;Ee];

end

%% EVAPORATOR MODEL
function [Tie,LMTDei]=evaporatormodel(nslices,T1,T2,Tels,T3,Pels,ref1)
%preallocations
Tie=zeros(2,nslices+1);
Tie(1,1)=T1;
Tie(1,end)=T2;
Tie(2,1)=Tels;
Tie(2,end)=T3;
Hie=zeros(1,nslices+1);
Hie(1)=refpropm('H','T',Tels,'P',Pels*100,ref1)/1000;
Hie(end)=refpropm('H','T',T3,'P',Pels*100,ref1)/1000;
LMTDei=zeros(1,nslices);

%intermediate enthalpy change
DHe=(Hie(1)-Hie(end))/nslices; %kJ/kg

for i=2:nslices
    %intermediate heat source temperature
    Tie(1,i)=Tie(1,i-1)-(T1-T2)/nslices; %K
    %intermediate refrigerant temperature
    Hie(i)=Hie(i-1)-DHe; %kJ/kg
    %intermediate refrigerant temperature
    Tie(2,i)=refpropm('T','P',Pels*100,'H',Hie(i)*1000,ref1); %K
end

for i=1:nslices
    %intermediate logarithmic mean temperature difference in the evaporator
    LMTDei(i)=((Tie(1,i+1)-Tie(2,i+1))-(Tie(1,i)-Tie(2,i)))/log((Tie(1,i+1)-Tie(2,i+1))/(Tie(1,i)-Tie(2,i))); %K
end
end

%% CASCADE HEAT EXCHANGER MODEL
function [Tiche,LMTDchei]=cascadeHXmodel(nslices,Tcls,T5,Pcls,ref1,T7,T8,Pehs,ref2)
%preallocations
Tiche=zeros(2,nslices+1);
Tiche(1,1)=Tcls;
Tiche(1,end)=T5;
Tiche(2,1)=T7;
Tiche(2,end)=T8;
Hiche=zeros(2,nslices);
Hiche(1,1)=refpropm('H','T',Tcls,'P',Pcls*100,ref1)/1000;
Hiche(1,end)=refpropm('H','T',T5,'P',Pcls*100,ref1)/1000;
Hiche(2,1)=refpropm('H','T',T7,'P',Pehs*100,ref2)/1000;
Hiche(2,end)=refpropm('H','T',T8,'P',Pehs*100,ref2)/1000;
LMTDchei=zeros(1,nslices);

%low stage intermediate enthalpy change
DHls=(Hiche(1,end)-Hiche(1,1))/nslices; %kJ/kg
%high stage intermediate enthalpy change
DHhs=(Hiche(2,end)-Hiche(2,1))/nslices; %kJ/kg

for i=2:nslices
    %low stage intermediate refrigerant enthalpy
    Hiche(1,i)=Hiche(1,i-1)+DHls; %kJ/kg
    %low stage intermediate refrigerant temperature
    Tiche(1,i)=refpropm('T','P',Pcls*100,'H',Hiche(1,i)*1000,ref1); %K
    %high stage intermediate refrigerant enthalpy
    Hiche(2,i)=Hiche(2,i-1)+DHhs; %kJ/kg
    %high stage intermediate refrigerant temperature
    Tiche(2,i)=refpropm('T','P',Pehs*100,'H',Hiche(2,i)*1000,ref2); %K
end

for i=1:nslices
    %intermediate logarithmic mean temperature difference in the cascade heat exchanger
    LMTDchei(i)=((Tiche(1,i+1)-Tiche(2,i+1))-(Tiche(1,i)-Tiche(2,i)))/log((Tiche(1,i+1)-Tiche(2,i+1))/(Tiche(1,i)-Tiche(2,i))); %K
end
end

%% CONDENSER MODEL
function [Tic,LMTDci]=condensermodel(nslices,T11,T12,T10b,T9,Pchs,ref2)
%preallocations
Tic=zeros(2,nslices+1);
Tic(1,1)=T11;
Tic(1,end)=T12;
Tic(2,1)=T10b;
Tic(2,end)=T9;
Hic=zeros(1,nslices+1);
Hic(1)=refpropm('H','T',T10b,'P',Pchs*100,ref2)/1000;
Hic(end)=refpropm('H','T',T9,'P',Pchs*100,ref2)/1000;
LMTDci=zeros(1,nslices);

%intermediate enthalpy change
DHc=(Hic(end)-Hic(1))/nslices; %kJ/kg

for i=2:nslices
    %intermediate hot water temperature
    Tic(1,i)=Tic(1,i-1)+(T12-T11)/nslices; %K
    %intermediate refrigerant enthalpy
    Hic(i)=Hic(i-1)+DHc; %kJ/kg
    %intermediate refrigerant temperature
    Tic(2,i)=refpropm('T','P',Pchs*100,'H',Hic(i)*1000,ref2); %K
end

for i=1:nslices
    %intermediate logarithmic mean temperature difference in the condenser
    LMTDci(i)=((Tic(2,i+1)-Tic(1,i+1))-(Tic(2,i)-Tic(1,i)))/log((Tic(2,i+1)-Tic(1,i+1))/(Tic(2,i)-Tic(1,i))); %K
end
end

%% INTERNAL HEAT EXCHANGERS MODEL
function [Tiihx,LMTDihxi]=internalHXmodel(nslices,T6,Tcls,Pcls,Tels,T4,Pels,ref1)
%preallocations
Tiihx=zeros(2,nslices+1);
Tiihx(1,1)=T6;
Tiihx(1,end)=Tcls;
Tiihx(2,1)=Tels;
Tiihx(2,end)=T4;
Hiihx=zeros(2,nslices+1);
Hiihx(1,1)=refpropm('H','T',T6,'P',Pcls*100,ref1)/1000;
Hiihx(1,end)=refpropm('H','P',Pcls*100,'Q',0,ref1)/1000;
Hiihx(2,1)=refpropm('H','P',Pels*100,'Q',1,ref1)/1000;
Hiihx(2,end)=refpropm('H','T',T4,'P',Pels*100,ref1)/1000;
LMTDihxi=zeros(1,nslices);

%low stage intermediate enthalpy change
DHiihxls=(Hiihx(1,end)-Hiihx(1,1))/nslices; %kJ/kg

for i=2:nslices
    %low stage intermediate hot refrigerant enthalpy
    Hiihx(1,i)=Hiihx(1,i-1)+DHiihxls; %kJ/kg
    %low stage intermediate hot refrigerant temperature
    Tiihx(1,i)=refpropm('T','P',Pcls*100,'H',Hiihx(1,i)*1000,ref1); %K
    %low stage intermediate cold refrigerant enthalpy
    Hiihx(2,i)=Hiihx(2,i-1)+DHiihxls; %kJ/kg
    %low stage intermediate cold refrigerant temperature
    Tiihx(2,i)=refpropm('T','P',Pels*100,'H',Hiihx(2,i)*1000,ref1); %K  
end

for i=1:nslices
    %low stage intermediate logarithmic mean temperature difference in the internal heat exchanger
    LMTDihxi(1,i)=((Tiihx(1,i+1)-Tiihx(2,i+1))-(Tiihx(1,i)-Tiihx(2,i)))/log((Tiihx(1,i+1)-Tiihx(2,i+1))/(Tiihx(1,i)-Tiihx(2,i))); %K
end
end