function [COP,Qc,Qe,Wc,T2a,T12]=heatpump_final(operation)

%% VARIABLE DESIGN AND PROCESS PARAMETERS
switch operation
    %full load operation
    case 'fl'
        %exchange area of the condenser
        Ac=22; %m^2
        %exchange area of the cascade heat exchanger
        Ache=12; %m^2
        %exchange area of the evaporator
        Ae=13; %m^2
        %number of operating compressors
        ncomp=2;
        %heat source mass flow rate
        m1=3.76; %kg/s
        %hot water mass flow rate
        m2=1.19; %kg/s
    %part load operation
    case 'pl'
        %exchange area of the condenser
        Ac=11; %m^2
        %exchange area of the cascade heat exchanger
        Ache=6; %m^2
        %exchange area of the evaporator
        Ae=6.5; %m^2
        %number of operating compressors
        ncomp=1;
        %heat source mass flow rate
        m1=1.88; %kg/s
        %hot water mass flow rate
        m2=0.6; %kg/s
end

%% FIXED DESIGN AND PROCESS PARAMETERS
%low stage refrigerant
ref1='co2butane 0.03-0.97.mix';
%high stage refrigerant
ref2='co2pentane 0.02-0.98.mix';
%heat source inlet temperature
T1=40+273.15; %K
%hot water inlet temperature
T11=60+273.15; %K
%low stage compressor size
Vcls=145; %m^3/h
%high stage compressor size
Vchs=150; %m^3/h
%degree of superheating
Tsup=12; %K
%temperature difference between the stages
Tdiff=8; %K

%% PROCESS CONSTANTS
%global heat transfer coefficient in the high stage condenser
Uc=1400; %W/m^2*K
%global heat transfer coefficient in the cascade heat exchanger
Uche=1800; %W/m^2*K
%global heat transfer coefficient in the low stage evaporator
Ue=1600; %W/m^2*K

%% INITIAL GUESS VALUES
%high stage condensation temperature
Tchs=80+273.15; %K
%low stage condensation temperature
Tcls=52+273.15; %K
%low stage evaporation temperature
Tels=30+273.15; %K

%% SYSTEM SOLUTION
%system equations solution
[Ts,E]=fsolve(@eqn,[Tchs,Tcls,Tels],[],Tsup,Tsup,Tdiff,T1,T11,m1,m2,Uc,Ac,Uche,Ache,Ue,Ae,Vcls,Vchs,ncomp,ref1,ref2);

%high stage condensation temperature
Tchs=Ts(1); %K
%low stage condensation temperature
Tcls=Ts(2); %K
%low stage evaporation temperature
Tels=Ts(3); %K
%error in the condenser LMTD
Ec=E(1); %K
%error in the cascade heat exchanger LMTD
Eche=E(2); %K
%error in the evaporator
Ee=E(3); %K

clc
if abs(Ec)>0.1 || abs(Eche)>0.1 || abs(Ee)>0.1
    error('LMTDs not in agreement')
else
    fprintf('Equations solved \n')
end

%% CYCLES COMPUTATIONS
%high stage condenser pressure
Pchs=refpropm('P','T',Tchs,'Q',0,ref2)/100; %bar
%low stage condenser pressure
Pcls=refpropm('P','T',Tcls,'Q',0,ref1)/100; %bar
%low stage evaporator pressure
Pels=refpropm('P','T',Tels,'Q',1,ref1)/100; %bar
%low stage compression ratio
rls=Pcls/Pels;
%low stage isoentropic efficiency
etaSls=-0.00000461*rls^6+0.00027131*rls^5-0.00628605*rls^4+0.07370258*rls^3-0.46054399*rls^2+1.40653347*rls-0.87811477;
%low stage volumetric efficiency
etaVls=0.001*rls^2-0.0487*rls+0.9979;

%low stage compressor inlet temperature
T4=Tels+Tsup; %K
%low stage compressor inlet enthalpy
H4=refpropm('H','T',T4,'P',Pels*100,ref1)/1000; %kJ/kg
%low stage compressor inlet density
D4=refpropm('D','T',T4,'P',Pels*100,ref1); %kg/m^3
%low stage compressor inlet entropy
S4=refpropm('S','T',T4,'P',Pels*100,ref1)/1000; %kJ/kg*K
%low stage condenser inlet enthalpy (isentropic)
H5s=refpropm('H','P',Pcls*100,'S',S4*1000,ref1)/1000; %kJ/kg
%low stage condenser inlet enthalpy (real)
H5=H4+(H5s-H4)/etaSls; %kJ/kg
%low stage condenser inlet temperature
T5=refpropm('T','P',Pcls*100,'H',H5*1000,ref1); %K
%low stage internal heat exchanger enthalpy change
DHihxls=H4-refpropm('H','T',Tels,'Q',1,ref1)/1000; %kJ/kg
%low stage valve inlet enthalpy
H6=refpropm('H','T',Tcls,'Q',0,ref1)/1000-DHihxls; %kJ/kg
%low stage evaporator inlet enthalpy
H3=H6; %kJ/kg

%low stage refrigerant mass flow rate
mrefls=D4*Vcls/3600*etaVls*ncomp; %kg/s

%low stage compressor power demand
Wcls=(H5-H4)*mrefls; %kW
%low stage evaporator capacity
Qe=(refpropm('H','P',Pels*100,'Q',1,ref1)/1000-H3)*mrefls; %kW
%cascade heat exchanger capacity
Qche=(H5-refpropm('H','P',Pcls*100,'Q',0,ref1)/1000)*mrefls; %kW

%heat source specific heat capacity
cp1=refpropm('C','T',T1,'P',20*100,'water')/1000; %kJ/kg*K
%heat source outlet temperature
T2=T1-Qe/cp1/m1; %K

%high stage compressor inlet temperature
T8=T5-Tdiff; %K
%high stage evaporation temperature
Tehs=T8-Tsup; %K

%high stage evaporator pressure
Pehs=refpropm('P','T',Tehs,'Q',1,ref2)/100; %bar
%high stage compression ratio
rhs=Pchs/Pehs;
%high stage isoentropic efficiency
etaShs=-0.00000461*rhs^6+0.00027131*rhs^5-0.00628605*rhs^4+0.07370258*rhs^3-0.46054399*rhs^2+1.40653347*rhs-0.87811477;
%high stage volumetric efficiency
etaVhs=0.001*rhs^2-0.0487*rhs+0.9979;

%high stage compressor inlet enthalpy
H8=refpropm('H','T',T8,'P',Pehs*100,ref2)/1000; %kJ/kg
%high stage compressor inlet density
D8=refpropm('D','T',T8,'P',Pehs*100,ref2); %kg/m^3
%high stage compressor inlet entropy
S8=refpropm('S','T',T8,'P',Pehs*100,ref2)/1000; %kJ/kg*K
%high stage condenser inlet enthalpy (isentropic)
H9s=refpropm('H','P',Pchs*100,'S',S8*1000,ref2)/1000; %kJ/kg
%high stage condenser inlet enthalpy (real)
H9=H8+(H9s-H8)/etaShs; %kJ/kg

%high stage refrigerant mass flow rate
mrefhs=D8*Vchs/3600*etaVhs*ncomp; %kg/s

%high stage evaporator inlet enthalpy
H7=refpropm('H','P',Pehs*100,'Q',1,ref2)/1000-Qche/mrefhs; %kJ/kg
%high stage valve inlet enthalpy
H10=H7; %kJ/kg
%high stage intermediate subcooling temperature
T10b=T11+5; %K

%high stage compressor power demand
Wchs=(H9-H8)*mrefhs; %kW
%high stage condenser capacity
Qc=(H9-refpropm('H','T',T10b,'P',Pchs*100,ref2)/1000)*mrefhs; %kW

%hot water specific heat capacity
cp2=refpropm('C','T',T11,'P',20*100,'water')/1000; %kJ/kg*K
%hot water outlet temperature
T12=T11+Qc/cp2/m2; %K

%overall compressor power demand
Wc=Wchs+Wcls; %kW
%overall coefficient of performance
COP=Qc/Wc;

%heat source outlet temperature after energy integration
T2a=T2+mrefhs*(refpropm('H','T',T10b,'P',Pchs*100,ref2)/1000-H10)/m1/cp1; %K
%evaporator capacity after energy integration
Qe=Qe-m1*cp1*(T2a-T2); %kW

end

%% CASCADE HEAT PUMP EQUATIONS
function err=eqn(Ts,Tsuphs,Tsupls,Tdiff,T1,T11,m1,m2,Uc,Ac,Uche,Ache,Ue,Ae,Vcls,Vchs,ncomp,ref1,ref2)

Tchs=Ts(1);
Tcls=Ts(2);
Tels=Ts(3);

%high stage condenser pressure
Pchs=refpropm('P','T',Tchs,'Q',0,ref2)/100; %bar
%low stage condenser pressure
Pcls=refpropm('P','T',Tcls,'Q',0,ref1)/100; %bar
%low stage evaporator pressure
Pels=refpropm('P','T',Tels,'Q',1,ref1)/100; %bar
%low stage compression ratio
rls=Pcls/Pels;
%low stage isoentropic efficiency
etaSls=-0.00000461*rls^6+0.00027131*rls^5-0.00628605*rls^4+0.07370258*rls^3-0.46054399*rls^2+1.40653347*rls-0.87811477;
%low stage volumetric efficiency
etaVls=0.001*rls^2-0.0487*rls+0.9979;

%low stage compressor inlet temperature
T4=Tels+Tsupls; %K
%low stage compressor inlet enthalpy
H4=refpropm('H','T',T4,'P',Pels*100,ref1)/1000; %kJ/kg
%low stage compressor inlet density
D4=refpropm('D','T',T4,'P',Pels*100,ref1); %kg/m^3
%low stage compressor inlet entropy
S4=refpropm('S','T',T4,'P',Pels*100,ref1)/1000; %kJ/kg*K
%low stage condenser inlet enthalpy (isentropic)
H5s=refpropm('H','P',Pcls*100,'S',S4*1000,ref1)/1000; %kJ/kg
%low stage condenser inlet enthalpy (real)
H5=H4+(H5s-H4)/etaSls; %kJ/kg
%low stage condenser inlet temperature
T5=refpropm('T','P',Pcls*100,'H',H5*1000,ref1); %K
%internal heat exchanger enthalpy change
DH=H4-refpropm('H','T',Tels,'Q',1,ref1)/1000; %kJ/kg
%low stage valve inlet enthalpy
H6=refpropm('H','T',Tcls,'Q',0,ref1)/1000-DH; %kJ/kg
%low stage evaporator inlet enthalpy
H3=H6; %kJ/kg
%low stage evaporator inlet temperature
T3=refpropm('T','P',Pels*100,'H',H3*1000,ref1); %K

%low stage refrigerant mass flow rate
mrefls=D4*Vcls*ncomp/3600*etaVls; %kg/s

%evaporator capacity
Qe=(refpropm('H','P',Pels*100,'Q',1,ref1)/1000-H3)*mrefls; %kW
%cascade heat exchanger capacity
Qche=(H5-refpropm('H','P',Pcls*100,'Q',0,ref1)/1000)*mrefls; %kW

%cold water specific heat capacity
cp1=refpropm('C','T',T1,'P',20*100,'water')/1000; %kJ/kg*K
%cold water outlet temperature
T2=T1-Qe/cp1/m1; %K
%logarithmic mean temperature difference in the low stage evaporator
LMTDe=Qe*1000/Ue/Ae; %K
%error in the low stage evaporator
Ee=LMTDe-((T1-Tels)-(T2-T3))/log((T1-Tels)/(T2-T3)); %K

%high stage compressor inlet temperature
T8=T5-Tdiff; %K
%high stage evaporation temperature
Tehs=T8-Tsuphs; %K

%high stage evaporator pressure
Pehs=refpropm('P','T',Tehs,'Q',1,ref2)/100; %bar
%high stage compression ratio
rhs=Pchs/Pehs;
%high stage isoentropic efficiency
etaShs=-0.00000461*rhs^6+0.00027131*rhs^5-0.00628605*rhs^4+0.07370258*rhs^3-0.46054399*rhs^2+1.40653347*rhs-0.87811477;
%high stage volumetric efficiency
etaVhs=0.001*rhs^2-0.0487*rhs+0.9979;

%high stage compressor inlet enthalpy
H8=refpropm('H','T',T8,'P',Pehs*100,ref2)/1000; %kJ/kg
%high stage compressor inlet density
D8=refpropm('D','T',T8,'P',Pehs*100,ref2); %kg/m^3
%high stage compressor inlet entropy
S8=refpropm('S','T',T8,'P',Pehs*100,ref2)/1000; %kJ/kg*K
%high stage condenser inlet enthalpy (isentropic)
H9s=refpropm('H','P',Pchs*100,'S',S8*1000,ref2)/1000; %kJ/kg
%high stage condenser inlet enthalpy (real)
H9=H8+(H9s-H8)/etaShs; %kJ/kg
%high stage condenser inlet temperature
T9=refpropm('T','P',Pchs*100,'H',H9*1000,ref2); %K

%high stage refrigerant mass flow rate
mrefhs=D8*Vchs*ncomp/3600*etaVhs; %kg/s

%high stage evaporator inlet enthalpy
H7=refpropm('H','P',Pehs*100,'Q',1,ref2)/1000-Qche/mrefhs; %kJ/kg
%high stage evaporator inlet temperature
T7=refpropm('T','P',Pehs*100,'H',H7*1000,ref2); %K

%condenser capacity
Qc=(H9-refpropm('H','P',Pchs*100,'Q',0,ref2)/1000)*mrefhs; %kW

%hot water specific heat capacity
cp2=refpropm('C','T',T11,'P',20*100,'water')/1000; %kJ/kg*K
%hot water outlet temperature
T12=T11+Qc/cp2/m2; %K
%logarithmic mean temperature difference in the high stage condenser
LMTDc=Qc*1000/Uc/Ac; %K
%error in the high stage condenser
Ec=LMTDc-((T9-T12)-(5))/log((T9-T12)/(5)); %K

%logarithmic mean temperature difference in the cascade heat exchanger
LMTDche=Qche*1000/Uche/Ache; %K
%error in the cascade heat exchanger
Eche=LMTDche-((T5-T8)-(Tcls-T7))/log((T5-T8)/(Tcls-T7)); %K

err=[Ec;Eche;Ee];

end