clear, clc, close all
set(0,'DefaultLineLineWidth',1.5)
addpath 'C:\Program Files (x86)\REFPROP\'
format bank

%% PROCESS PARAMETERS
%heat pump capacity
Qc=200; %kW

%low stage refrigerant
ref1='co2butane 0.03-0.97.mix';
%high stage refrigerant
ref2='co2pentane 0.02-0.98.mix';

%isoentropic efficiency function
etaS=@(r)-0.00000461*r^6+0.00027131*r^5-0.00628605*r^4+0.07370258*r^3-0.46054399*r^2+1.40653347*r-0.87811477;
%volumetric efficiency function
etaV=@(r)0.0011*r^2-0.0487*r+0.9979;

%low stage superheating degree
Tsup=12; %K
%temperature difference between the stages
Tdiff=8; %K

%global heat transfer coefficient in the internal heat exchanger
Uihx=1000; %W/m^2*K
%global heat transfer coefficient in the evaporator
Ue=1600; %W/m^2*K
%global heat transfer coefficient in the cascade heat exchanger
Uche=1800; %W/m^2*K
%global heat transfer coefficient in the condenser
Uc=1400; %W/m^2*K

%heat source inlet temperature
T1=40+273.15; %K
%heat source outlet temperature
T2=30+273.15; %K
%hot water inlet temperature
T11=60+273.15; %K
%hot water outlet temperature
T12=100+273.15; %K

%% LOW STAGE CYCLE COMPUTATIONS
%low stage evaporation pressure
Pels=3; %bar
%low stage evaporation temperature
T4a=refpropm('T','P',Pels*100,'Q',1,ref1); %K

%low stage compressor inlet temperature
T4=T4a+Tsup; %K
%low stage compressor inlet enthalpy
H4=refpropm('H','T',T4,'P',Pels*100,ref1)/1000; %kJ/kg
%low stage compressor inlet entropy
S4=refpropm('S','T',T4,'P',Pels*100,ref1)/1000; %kJ/kg*K
%low stage compressor inlet density
D4=refpropm('D','T',T4,'P',Pels*100,ref1); %kg/m^3

%low stage condensation pressure
Pcls=9; %bar
%low stage condensation temperature
T6a=refpropm('T','P',Pcls*100,'Q',0,ref1); %K

%low stage compression ratio
rls=Pcls/Pels;
%low stage condenser inlet enthalpy (isoentropic)
H5s=refpropm('H','P',Pcls*100,'S',S4*1000,ref1)/1000; %kJ/kg
%low stage condenser inlet enthalpy (real)
H5=H4+(H5s-H4)/etaS(rls); %kJ/kg
%low stage condenser inlet temperature
T5=refpropm('T','P',Pcls*100,'H',H5*1000,ref1); %K
%low stage internal heat exchanger enthalpy change
DHihxls=H4-refpropm('H','P',Pels*100,'Q',1,ref1)/1000; %kJ/kg
%low stage valve inlet enthalpy
H6=refpropm('H','P',Pcls*100,'Q',0,ref1)/1000-DHihxls; %kJ/kg
%low stage valve inlet temperature
T6=refpropm('T','P',Pcls*100,'H',H6*1000,ref1); %K
%low stage subcooling degree
Tsubls=T6a-T6; %K
%low stage evaporator inlet enthalpy
H3=H6; %kJ/kg
%low stage evaporator inlet temperature
T3=refpropm('T','P',Pels*100,'H',H3*1000,ref1); %K

%% HIGH STAGE CYCLE COMPUTATIONS
%high stage evaporator outlet temperature
T8=T5-Tdiff; %K
%high stage compressor inlet temperature
T8a=T8-Tsup; %K
%high stage evaporation pressure
Pehs=refpropm('P','T',T8a,'Q',1,ref2)/100; %bar

%high stage compressor inlet enthalpy
H8=refpropm('H','T',T8,'P',Pehs*100,ref2)/1000; %kJ/kg
%high stage compressor inlet entropy
S8=refpropm('S','T',T8,'P',Pehs*100,ref2)/1000; %kJ/kg*K
%high stage compressor inlet density
D8=refpropm('D','T',T8,'P',Pehs*100,ref2); %kg/m^3

%high stage condensation pressure
Pchs=7; %bar
%high stage condensation temperature
T10b=refpropm('T','P',Pchs*100,'Q',0,ref2); %K

%high stage compression ratio
rhs=Pchs/Pehs;
%high stage condenser inlet enthalpy (isoentropic)
H9s=refpropm('H','P',Pchs*100,'S',S8*1000,ref2)/1000; %kJ/kg
%high stage condenser inlet enthalpy (real)
H9=H8+(H9s-H8)/etaS(rhs); %kJ/kg
%high stage condenser inlet temperature
T9=refpropm('T','P',Pchs*100,'H',H9*1000,ref2); %K
%high stage evaporator inlet enthalpy
H7=H8-H5+refpropm('H','P',Pcls*100,'Q',0,ref1)/1000; %kJ/kg
%high stage evaporator inlet temperature
T7=refpropm('T','P',Pehs*100,'H',H7*1000,ref2); %K
%high stage valve inlet enthalpy
H10=H7; %kJ/kg
%high stage valve inlet temperature
T10=refpropm('T','P',Pchs*100,'H',H10*1000,ref2); %K
%high stage subcooling temperature
T10a=T11+5; %K
%high stage internal heat exchanger enthalpy change
DHihxhs=refpropm('H','T',T10a,'P',Pchs*100,ref2)/1000-H10; %kJ/kg
%high stage degree of superheating
Tsubhs=T10b-T10; %K

%% EXCHANGERS CAPACITIES
%high stage refrigerant mass flow rate
mrefhs=Qc/(H9-refpropm('H','T',T10a,'P',Pchs*100,ref2)/1000); %kg/s
%cascade heat exchanger capacity
Qche=mrefhs*(H8-H7); %kW
%low stage refrigerant mass flow rate
mrefls=Qche/(H5-refpropm('H','P',Pcls*100,'Q',0,ref1)/1000); %kg/s
%evaporator capacity
Qe=mrefls*(refpropm('H','P',Pels*100,'Q',1,ref1)/1000-H3); %kW

%% COMPRESSION WORKS
%low stage compressor power
Wcls=mrefls*(H5-H4); %kW
%high stage compressor power
Wchs=mrefhs*(H9-H8); %kW

%% INLET/OUTLET FLOWS
%heat source mass flow rate
m1=Qe/refpropm('C','T',(T1+T2)/2,'P',20*100,'water')/(T1-T2)*1000; %kg/s
%hot water mass flow rate
m2=Qc/refpropm('C','T',(T11+T12)/2,'P',20*100,'water')/(T12-T11)*1000; %kg/s
%heat source outlet temperature new
T2a=T2+mrefhs/m1*DHihxhs/refpropm('C','T',(T1+T2)/2,'P',20*100,'water')*1000; %K

%% LOGARITHMIC MEAN TEMPERATURE DIFFERENCES
%condenser inlet temperature difference
thetacin=T9-T12; %K
%condenser outlet temperature difference
thetacout=T10a-T11; %K
%condenser logarithmic mean temperature difference
LMTDc=(thetacin-thetacout)/log(thetacin/thetacout); %K

%cascade heat exchanger inlet temperature difference
thetachein=T5-T8a; %K
%cascade heat exchanger outlet temperature difference
thetacheout=T6a-T7; %K
%cascade heat exchanger logarithmic mean temperature difference
LMTDche=(thetachein-thetacheout)/log(thetachein/thetacheout); %K

%evaporator inlet temperature difference
thetaein=T1-T4a; %K
%evaporator outlet temperature difference
thetaeout=T2-T3; %K
%evaporator logarithmic mean temperature difference
LMTDe=(thetaein-thetaeout)/log(thetaein/thetaeout); %K

%low stage internal heat exchanger logarithmic mean temperature difference
LMTDihxls=(Tsup-Tsubls)/log(Tsup/Tsubls); %K
%high stage internal heat exchanger logarithmic mean temperature difference
LMTDihxhs=((T10-T2)-(T10a-T2a))/log((T10-T2)/(T10a-T2a)); %K

%% EXCHANGERS AREAS
%condenser area
Ac=Qc*1000/Uc/LMTDc; %m^2
%cascade heat exchanger area
Ache=Qche*1000/Uche/LMTDche; %m^2
%evaporator area
Ae=Qe*1000/Ue/LMTDe; %m^2
%low stage internal heat exchanger area
Aihxls=mrefls*DHihxls*1000/Uihx/LMTDihxls; %m^2
%high stage internal heat exchanger area
Aihxhs=mrefhs*DHihxhs*1000/Uihx/LMTDihxhs; %m^2

%% COMPRESSOR SIZES
%low stage compressor size
Vcls=3600*mrefls/D4/etaV(rls); %m^3/h
%high stage compressor size
Vchs=3600*mrefhs/D8/etaV(rhs); %m^3/h

%% COEFFICIENTS OF PERFORMACE
%low stage COP
COPls=Qche/Wcls;
%high stage COP
COPhs=Qc/Wchs;
%global COP
COP=Qc/(Wchs+Wcls);

%% SIZING RESULTS DISPLAY
sz=[7,3];
varTypes=["string","double","string"];
varNames=["Sizing Parameter";"Value";"Unit"];
Results1=table('Size',sz,'VariableTypes',varTypes,'VariableNames',varNames);
Parameters=["Evaporator Area";"Cascade HX Area";"Condenser Area";"LS Internal HX Area";"HS Internal HX Area";"LS Compressor Size";"HS Compressor Size"];
Values=[Ae;Ache;Ac;Aihxls;Aihxhs;Vcls;Vchs];
Units=["m^2";"m^2";"m^2";"m^2";"m^2";"m^3/h";"m^3/h"];
Results1{:,1}=Parameters;
Results1{:,2}=Values;
Results1{:,3}=Units;
disp(Results1)

%% OPERATION RESULTS DISPLAY
sz=[5,3];
varTypes=["string","double","string"];
varNames=["Operation Parameter";"Value";"Unit"];
Results2=table('Size',sz,'VariableTypes',varTypes,'VariableNames',varNames);
Parameters=["Heat Source Inlet Temperature";"Hot Water Inlet Temperature";"Heat Source Mass Flow Rate";"Hot Water Mass Flow Rate";"COP"];
Values=[T1-273.15;T11-273.15;m1;m2;COP];
Units=["°C";"°C";"kg/s";"kg/s";"[-]"];
Results2{:,1}=Parameters;
Results2{:,2}=Values;
Results2{:,3}=Units;
disp(Results2)

%% T-H DIAGRAM
figure
hold on
plot([refpropm('H','P',Pels*100,'Q',1,ref1)/1000,H3],[T1,T2]-273.15,'g-')
plot([H3,refpropm('H','P',Pels*100,'Q',1,ref1)/1000],[T3,T4a]-273.15,'b-')
plot([refpropm('H','P',Pels*100,'Q',1,ref1)/1000,H4],[T4a,T4]-273.15,'b-')
plot([H4,H5],[T4,T5]-273.15,'b-')
plot([H5,refpropm('H','P',Pcls*100,'Q',1,ref1)/1000],[T5,refpropm('T','P',Pcls*100,'Q',1,ref1)]-273.15,'b-')
plot([refpropm('H','P',Pcls*100,'Q',1,ref1),refpropm('H','P',Pcls*100,'Q',0,ref1)]/1000,[refpropm('T','P',Pcls*100,'Q',1,ref1),T6a]-273.15,'b-')
plot([refpropm('H','P',Pcls*100,'Q',0,ref1)/1000,H6],[T6a,T6]-273.15,'b-')
plot([H6,H3],[T6,T3]-273.15,'b-')
tr=H6-H7;
plot([H7,refpropm('H','P',Pehs*100,'Q',1,ref2)/1000]+tr,[T7,T8a]-273.15,'c-')
plot([refpropm('H','P',Pehs*100,'Q',1,ref2)/1000,H8]+tr,[T8a,T8]-273.15,'c-')
plot([H8,H9]+tr,[T8,T9]-273.15,'c-')
plot([H9,refpropm('H','P',Pchs*100,'Q',1,ref2)/1000]+tr,[T9,refpropm('T','P',Pchs*100,'Q',1,ref2)]-273.15,'c-')
plot([refpropm('H','P',Pchs*100,'Q',1,ref2),refpropm('H','P',Pchs*100,'Q',0,ref2)]/1000+tr,[refpropm('T','P',Pchs*100,'Q',1,ref2),T10b]-273.15,'c-')
plot([refpropm('H','P',Pchs*100,'Q',0,ref2)/1000,H10]+tr,[T10b,T10]-273.15,'c-')
plot([H10,H7]+tr,[T10,T7]-273.15,'c-')
plot([refpropm('H','T',T10a,'P',Pchs*100,ref2)/1000,H9]+tr,[T11,T12]-273.15,'r-')
scatter(H3,T3-273.15,'ko','filled')
scatter(refpropm('H','P',Pels*100,'Q',1,ref1)/1000,T4a-273.15,'ko','filled')
scatter(H4,T4-273.15,'ko','filled')
scatter(H5,T5-273.15,'ko','filled')
scatter(refpropm('H','P',Pcls*100,'Q',0,ref1)/1000,T6a-273.15,'ko','filled')
scatter(H6,T6-273.15,'ko','filled')
scatter(H7+tr,T7-273.15,'ko','filled')
scatter(refpropm('H','P',Pehs*100,'Q',1,ref2)/1000+tr,T8a-273.15,'ko','filled')
scatter(H8+tr,T8-273.15,'ko','filled')
scatter(H9+tr,T9-273.15,'ko','filled')
scatter(refpropm('H','P',Pchs*100,'Q',0,ref2)/1000+tr,T10b-273.15,'ko','filled')
scatter(refpropm('H','T',T10a,'P',Pchs*100,ref2)/1000+tr,T10a-273.15,'ko','filled')
scatter(H10+tr,T10-273.15,'ko','filled')
grid on
xlabel('Specific Enthalpy [kJ/kg]')
ylabel('Temperature [°C]')
title('T-H Diagram Cascade HP')
legend('Heat Source','Low stage','','','','','','','High Stage','','','','','','','Hot Water','Location','NorthWest')
text(H3-15,T3-273.15+3,cellstr('3'))
text(refpropm('H','P',Pels*100,'Q',1,ref1)/1000-15,T4a-273.15+3,cellstr('4a'))
text(H4-15,T4-273.15+3,cellstr('4'))
text(H5-15,T5-273.15+3,cellstr('5'))
text(refpropm('H','P',Pcls*100,'Q',0,ref1)/1000-15,T6a-273.15+3,cellstr('6a'))
text(H6-15,T6-273.15+3,cellstr('6'))
text(H7+tr-15,T7-273.15+3,cellstr('7'))
text(refpropm('H','P',Pehs*100,'Q',1,ref2)/1000+tr-15,T8a-273.15+3,cellstr('8a'))
text(H8+tr-15,T8-273.15+3,cellstr('8'))
text(H9+tr-15,T9-273.15+3,cellstr('9'))
text(refpropm('H','P',Pchs*100,'Q',0,ref2)/1000+tr-15,T10b-273.15+3,cellstr('10a'))
text(refpropm('H','T',T10a,'P',Pchs*100,ref2)/1000+tr-15,T10a-273.15+3,cellstr('10b'))
text(H10+tr-15,T10-273.15+3,cellstr('10'))
xlim([H3-100,H9+tr+100])